<?php

namespace Drupal\cck_select_other;

use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * A trait containing method for getting entity displays from a field definition
 * and extracting allowed values from it.
 */
trait EntityDisplayTrait {

  /**
   * Get the entity form displays for a field definition.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition to use.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entityManager
   *   The entity manager for getting entities from storage.
   * @return \Drupal\Core\Entity\Display\EntityFormDisplayInterface[]|null
   *   An array of entity form displays.
   */
  protected function getFormDisplays(FieldDefinitionInterface $definition, EntityManagerInterface $entityManager) {

    $params = [
      'targetEntityType' => $definition->getTargetEntityTypeId()
    ];

    return $entityManager->getStorage('entity_form_display')->loadByProperties($params);
  }

  /**
   * Check to see if a field has the select other widget configured. This is all
   * because of loose coupling makes it difficult to figure out what's what.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition to check.
   * @return boolean
   *   TRUE if the definition has a select other widget defined in one of its
   * form displays.
   */
  public function hasSelectOtherWidget(FieldDefinitionInterface $definition) {
    $displays = $this->getFormDisplays($definition, \Drupal::entityManager());
    $field_name = $definition->getName();

    return array_reduce($displays, function(&$result, $display) use ($field_name) {
      /** @var \Drupal\Core\Entity\Display\EntityFormDisplayInterface $display */
      if (!$result && $display->getRenderer($field_name) !== NULL) {
        $widget_id = $display->getRenderer($field_name)->getPluginId();
        if ($widget_id === 'cck_select_other') {
          $result = TRUE;
        }
      }
      return $result;
    }, FALSE);
  }

  /**
   * Get the select other widget settings from the form display.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $definition
   *   The field definition to extract the widget from.
   * @return array
   *   An array of settings.
   */
  public function getWidgetSettings(FieldDefinitionInterface $definition) {
    $displays = $this->getFormDisplays($definition, \Drupal::entityManager());
    $field_name = $definition->getName();
    $settings = [];

    $widget = array_reduce($displays, function(&$result, $display) use ($field_name) {
      /** @var \Drupal\Core\Entity\Display\EntityFormDisplayInterface $display */
      if (!$result) {
        $widget = $display->getRenderer($field_name);
        if ($widget && $widget->getPluginId() === 'cck_select_other') {
          $result = $widget;
        }
      }
      return $result;
    }, FALSE);

    if ($widget) {
      $settings = $widget->getSettings();
    }

    return $settings;
  }

}